#!/opt/LWperl/bin/perl -w
###
### Snmpcmds: a front-end to the perl snmp modules
### By Blaine Bauer, bbauer@netlane.com
###
### The vast majority of this module is based on code from other authors in the initial 
### SNMP_Session distribution
###
### No warranty is implied or will be given with this module.  Use at your own risk.
###
## formats:
##   ($values, $oids)  = snmpwalk($host, $community, $oid, <$stopval>);
##   ($values, $oids)  = snmpwalkhex($host, $community, $oid, <$stopval>);
##   ($response) = snmpget($host, $community, $oid)
##   ($response) = snmpgethex($host, $community, $oid);
##   ($response) = &snmpset($host, $community, $oid, $type, $value);
##   $colon_notation = ether_hex (hex_string ($value));

# USAGE HINTS:
#  ** Walk commands return two scalar array references; using the above format:
#   	$$values[n]  is the value for the oid stored in  $$oids[n]
#   	The array for the values is @$values
#   	The array for the oids is @$oids
#  ** Walk commands also have a "stop value", a value which, if found, stops the walk
#     and returns only two scalars, the value and the oid. This can make a walk quicker
#     if you know the value and need the oid, e.g. for indices

# Snmpset - type is 'string' 'int' (i.e. integer)
# To set hex values:
# 1. IP address from hostname: 
#	($hname,$haliases,$haddrtype,$hlenght,@haddrs)= gethostbyname($remote_hostname);
#		my $binhexaddr = $haddrs[0];
# 2. IP address from dotted notation:
#	($ip1,$ip2,$ip3,$ip4) = split ('\.',$remote_hostname);
#	$hexaddr = sprintf("%2x%2x%2x%2x",$ip1,$ip2,$ip3,$ip4);
#	$hexaddr =~ tr/ /0/;
#	my $binhexaddr = pack("H8", $hexaddr);

package snmpcmd2;		

require 5.002;

use strict;
use BER;
use SNMP_Session;

use vars qw(@ISA @EXPORT);
use Exporter;

@ISA = qw(Exporter);

@EXPORT = qw(snmpset snmpget snmpwalk);

my %OIDS = (
            'system'       => '1.3.6.1.2.1.1',
            'sysDescr'     => '1.3.6.1.2.1.1.1',
            'sysUptime'    => '1.3.6.1.2.1.1.3',
            'sysContact'   => '1.3.6.1.2.1.1.4',
            'sysLocation'  => '1.3.6.1.2.1.1.6',

	 );


sub snmpwalk {
#format: snmpwalk($host, $community, $oid <, stopvalue> );

my @inarg=@_;
my $hostname = $inarg[0];
my $community = $inarg[1];
my @raw_oid = split ('\.', $inarg[2]);
my $stopval = $inarg[3] || "null";
my $retval;
my $retoid;
my $session;

die "Couldn't open SNMP session to $hostname"
    unless ($session = SNMP_Session->open ($hostname, $community, 161));

my $enc_oid = encode_oid (@raw_oid);
my @oids = ($enc_oid);
my @next_oids;
my $oid;
my $i;
for (;;) {
    if ($session->getnext_request_response (@oids)) {
	my $response = $session->pdu_buffer;
	my ($bindings, $binding, $oid, $value);

	($bindings) = $session->decode_get_response ($response);
	@next_oids = ();

	($binding,$bindings) = decode_sequence ($bindings);
	($oid,$value) = decode_by_template ($binding, "%O%@");
	last
	    unless BER::encoded_oid_prefix_p ($enc_oid, $oid);
	push @next_oids, $oid;
	#print pretty_print ($value), ' [',pretty_print ($oid), "]\n";
	push (@{$retval}, pretty_print($value)); push (@{$retoid}, pretty_print($oid));
	last if (pretty_print($value) EQ $stopval);	

    } else {
	die "No response received.\n";
    }
    @oids = @next_oids;
}

$session->close ();

1;
if ($stopval EQ "null") {return ($retval, $retoid);}
   else {return( pop(@{$retval}),pop(@{$retoid}) );}
}

sub snmpwalkhex {
#format: snmpwalkhex($host, $community, $oid);
my @inarg=@_;
my $hostname = $inarg[0];
my $community = $inarg[1];
my $stopval = $inarg[3] || "null";
my @raw_oid = split ('\.', $inarg[2]);
my $retval;
my $retoid;
my $session;

die "Couldn't open SNMP session to $hostname"
    unless ($session = SNMP_Session->open ($hostname, $community, 161));

my $enc_oid = encode_oid (@raw_oid);
my @oids = ($enc_oid);
my @next_oids;
my $oid;
my $i;
for (;;) {
    if ($session->getnext_request_response (@oids)) {
	my $response = $session->pdu_buffer;
	my ($bindings, $binding, $oid, $value);

	($bindings) = $session->decode_get_response ($response);
	@next_oids = ();

	($binding,$bindings) = decode_sequence ($bindings);
	($oid,$value) = decode_by_template ($binding, "%O%@");
	last
	    unless BER::encoded_oid_prefix_p ($enc_oid, $oid);
	push @next_oids, $oid;
	#print hex_string ($value), ' [',pretty_print ($oid), "]\n";
	push (@{$retval}, hex_string($value)); push (@{$retoid}, pretty_print($oid));
	last if (hex_string($value) EQ $stopval);	

    } else {
	die "No response received.\n";
    }
    @oids = @next_oids;
}

$session->close ();

1;
if ($stopval EQ "null") {return ($retval, $retoid);}
   else {return( pop(@{$retval}),pop(@{$retoid}) );}
}

sub snmpget {
# format: ($response) = snmpget($host, $community, $oid)
	my($host,$community,@varList) = @_;
	my(@enoid, $var,$response, $bindings, $binding, $value, $inoid,
		$outoid, $upoid,$oid,@retvals);
	grep ($_=toOID($_), @varList);
	srand();
	my $session = SNMP_Session->open ($host , $community, 161);
	if ($session->get_request_response(@varList)) {
		$response = $session->pdu_buffer;
		($bindings) = $session->decode_get_response ($response);
		$session->close ();
		while ($bindings) {
			($binding,$bindings) = decode_sequence ($bindings);
			($oid,$value) = decode_by_template ($binding, "%O%@");
			my $tempo = pretty_print($value);
			$tempo=~s/\t/ /g;
			$tempo=~s/\n/ /g;
			$tempo=~s/^\s+//;
			$tempo=~s/\s+$//;
			push @retvals,  $tempo;
		}
		return (@retvals);
	} else {
		return (-1,-1);
	}
}

sub snmpgethex {
# format: snmpgethex($host, $community, $oid)
	my($host,$community,@varList) = @_;
	my(@enoid, $var,$response, $bindings, $binding, $value, $inoid,
		$outoid, $upoid,$oid,@retvals);
	grep ($_=toOID($_), @varList);
	srand();
	my $session = SNMP_Session->open ($host , $community, 161);
	if ($session->get_request_response(@varList)) {
		$response = $session->pdu_buffer;
		($bindings) = $session->decode_get_response ($response);
		$session->close ();
		while ($bindings) {
			($binding,$bindings) = decode_sequence ($bindings);
			($oid,$value) = decode_by_template ($binding, "%O%@");
#			my $tempo = ether_hex (hex_string ($value));
			my $tempo = hex_string ($value);
			push @retvals,  $tempo;
		}
		return (@retvals);
	} else {
		return (-1,-1);
	}
}

sub snmpset {
# format ($response) = &snmpset($host, $community, $oid, $type, $value);
# type is 'string' or 'integer'
	my($host,$community,@varList) = @_;
	my(@enoid, $response, $bindings, $binding, $inoid,$outoid,
		$upoid,$oid,@retvals);
  	my ($type,$value);
	while (@varList) {
		$oid   = toOID(shift @varList);
		$type  = shift @varList;
		$value = shift @varList;
		($type eq 'string') && do {
			$value = encode_string($value);
			push @enoid, [$oid,$value];
			next;
		};
		($type eq 'int') && do {
			$value = encode_int($value);
			push @enoid, [$oid,$value];
			next;
		};
		die "Unknown SNMP type: $type";
	}
	srand();
	my $session = SNMP_Session->open ($host , $community, 161);
	if ($session->set_request_response(@enoid)) {
		$response = $session->pdu_buffer;
		($bindings) = $session->decode_get_response ($response);
		$session->close ();
		while ($bindings) {
			($binding,$bindings) = decode_sequence ($bindings);
			($oid,$value) = decode_by_template ($binding, "%O%@");
			my $tempo = pretty_print($value);
			$tempo=~s/\t/ /g;
			$tempo=~s/\n/ /g;
			$tempo=~s/^\s+//;
			$tempo=~s/\s+$//;
			push @retvals,  $tempo;
		}
		return (@retvals);
	} else {
		return (-1,-1);
	}
}

sub toOID {
	my $var = shift;
	if ($var =~ /^([a-z]+[^\.]*)/i) {
		my $oid = $OIDS{$1};
		if ($oid) {
			$var =~ s/$1/$oid/;
		} else {
			die "Unknown SNMP var $var\n"
		}
	}
	encode_oid((split /\./, $var));
}

## ether_hex (HEX_STRING)
##
## Converts a raw hex representation into the common form used in
## Ethernet addresses, e.g. "080020830069" becomes
## "08:00:20:83:00:69".
##
sub ether_hex ($ )
{
    my ($string) = @_;
    $string =~ s/([0-9a-f][0-9a-f])/$1:/g;
    $string =~ s/:$//;
    $string;
}
